@@ -25,25 +25,22 @@ SEXP ema (SEXP x, SEXP n, SEXP ratio) {
     /* Initalize loop and PROTECT counters */
     int i, P=0;
 
-    /* assure that 'x' is double */
+    /* ensure that 'x' is double */
     if(TYPEOF(x) != REALSXP) {
       PROTECT(x = coerceVector(x, REALSXP)); P++;
     }
-    /* assure that 'n' is integer */
-    if(TYPEOF(n) != INTSXP) {
-      PROTECT(n = coerceVector(n, INTSXP)); P++;
-    }
 
     /* Pointers to function arguments */
     double *d_x = REAL(x);
-    int i_n = INTEGER(n)[0];
-    double d_ratio = REAL(ratio)[0];
+    int i_n = asInteger(n);
+    double d_ratio = asReal(ratio);
     
     /* Input object length */
     int nr = nrows(x);
 
     /* Initalize result R object */
-    SEXP result; PROTECT(result = allocVector(REALSXP,nr)); P++;
+    SEXP result;
+    PROTECT(result = allocVector(REALSXP,nr)); P++;
     double *d_result = REAL(result);
 
     /* Find first non-NA input value */
@@ -80,29 +77,26 @@ SEXP evwma (SEXP pr, SEXP vo, SEXP n) {
     /* Initalize loop and PROTECT counters */
     int i, P=0;
 
-    /* assure that 'pr' is double */
+    /* ensure that 'pr' is double */
     if(TYPEOF(pr) != REALSXP) {
       PROTECT(pr = coerceVector(pr, REALSXP)); P++;
     }
-    /* assure that 'vo' is double */
+    /* ensure that 'vo' is double */
     if(TYPEOF(vo) != REALSXP) {
       PROTECT(vo = coerceVector(vo, REALSXP)); P++;
     }
-    /* assure that 'n' is integer */
-    if(TYPEOF(n) != INTSXP) {
-      PROTECT(n = coerceVector(n, INTSXP)); P++;
-    }
 
     /* Pointers to function arguments */
     double *d_pr = REAL(pr);
     double *d_vo = REAL(vo);
-    int i_n = INTEGER(n)[0];
+    int i_n = asInteger(n);
     
     /* Input object length */
     int nr = nrows(pr);
 
     /* Initalize result R object */
-    SEXP result; PROTECT(result = allocVector(REALSXP,nr)); P++;
+    SEXP result;
+    PROTECT(result = allocVector(REALSXP,nr)); P++;
     double *d_result = REAL(result);
 
     /* Volume Sum */
@@ -139,3 +133,59 @@ SEXP evwma (SEXP pr, SEXP vo, SEXP n) {
     return(result);
 }
 
+SEXP vma (SEXP x, SEXP w, SEXP ratio) {
+    
+    /* Initalize loop and PROTECT counters */
+    int i, P=0;
+
+    /* ensure that 'x' is double */
+    if(TYPEOF(x) != REALSXP) {
+      PROTECT(x = coerceVector(x, REALSXP)); P++;
+    }
+    /* ensure that 'w' is double */
+    if(TYPEOF(w) != REALSXP) {
+      PROTECT(w = coerceVector(w, REALSXP)); P++;
+    }
+
+    /* Pointers to function arguments */
+    double *d_x = REAL(x);
+    double *d_w = REAL(w);
+    double d_ratio = asReal(ratio);
+    
+    /* Input object length */
+    int nr = nrows(x);
+
+    /* Initalize result R object */
+    SEXP result;
+    PROTECT(result = allocVector(REALSXP,nr)); P++;
+    double *d_result = REAL(result);
+
+    /* Find first non-NA input value */
+    int beg = 0;
+    d_result[beg] = 0;
+    for(i = 0; i <= beg; i++) {
+        /* Account for leading NAs in input */
+        if(ISNA(d_x[i]) || ISNA(d_w[i])) {
+            d_result[i] = NA_REAL;
+            beg++;
+            d_result[beg] = 0;
+            continue;
+        }
+        /* Set leading NAs in output */
+        if(i < beg) {
+            d_result[i] = NA_REAL;
+        }
+        /* Raw mean to start VMA */
+        d_result[beg] += d_x[i];
+    }
+
+    /* Loop over non-NA input values */
+    for(i = beg+1; i < nr; i++) {
+        d_result[i] = d_x[i] * d_w[i] * d_ratio + d_result[i-1] * (1-d_ratio*d_w[i]);
+    }
+
+    /* UNPROTECT R objects and return result */
+    UNPROTECT(P);
+    return(result);
+}
+
